import os


def get_filename(filetype):
    '''
    forces user to input filename if one not detected with filetype == <filetype>

    <filetype> example: '.json'
    '''

    def outer(func):

        def inner(*args, **kwargs):
            print(kwargs)
            print(args)

            filename = kwargs.get('filename')
            if not filename:
                while True:
                    filename = input(
                        f'Please enter a valid filename (filetype== {filetype}: ')
                    if get_file_parts(filename)[1] == filetype:
                        kwargs['filename'] = filename

            return func(*args, **kwargs)

        return inner

    return outer


# region HELPERS FOR DECORATORS

def get_file_parts(file_):
    '''
    file needs appropriate path headed

    returns filename, file_extension
    '''
    return os.path.splitext(file_)


# endregion HELPERS FOR DECORATORS



from decorators import exponential_backoff
from profilers import cProfile_dec

# region *start* EXPONENTIAL BACKOFF PROOF/TESTING

i = 0
dict_ = {4: 'hey'}


@exponential_backoff(rate_gain=4, max_time=4)
def myfunc(a, b, c='this'):
    global i, dict_
    print(i)
    i += 1
    print(dict_[i])
    return i

# endregion *end* EXPONENTIAL BACKOFF PROOF/TESTING


@cProfile_dec()
def prof_test():
    j = 1
    for i in range(10000000):
        j *= 1.00000001

    return j



import traceback
import logging
import os


def log_error(filename, full_traceback=False):
    """ I don't think this decorator is currently functional """
  def outer(func):
    def inner(*args, **kwargs):
      try:
        return func(*args, **kwargs)
      except Exception as e:
        if not os.path.exists(f"/var/log/usr/{filename}"):
          with open(f"/var/log/usr/{filename}", 'w+'):
            pass

        logging.basicConfig(filename=f"/var/log/usr/{filename}", level=logging.INFO, format='%(asctime)s %(message)s')
        if full_traceback:
          tb = traceback.format_exc(limit=6).replace(",", "\n")
          logging.error(f'Full traceback: {tb}')
        else:
          logging.error(f'Error: {e}')
        print(e)
      return
    return inner
  return outer



import cProfile
import time


def cProfile_dec(sort_by='tottime'):

    def outer(func):

        def inner(*args, **kwargs):

            profile = cProfile.Profile()
            profile.enable()

            output = func(*args, **kwargs)

            profile.print_stats(sort=sort_by)
            # sorting methods here:
            # https://docs.python.org/3/library/profile.html#pstats.Stats.sort_stats

            profile.disable()
            return output

        return inner

    return outer


def timeit(func):
    # decorator that wraps function with simple time log

    def inner(*args, **kwargs):

        t0 = time.time()
        output = func(*args, **kwargs)
        print(f"Time taken for {func.__name__}", time.time() - t0)

        return output

    return inner
